@@ -1,10 +1,11 @@
 import os
-from flask import jsonify, request
-from bobaserver import app
-from .util import read_csv, read_json, read_key_safe, group_by, remove_na
 import numpy as np
 import pandas as pd
 import math
+from flask import jsonify, request
+from bobaserver import app
+from .util import read_csv, read_json, read_key_safe, group_by, remove_na
+import bobaserver.common as common
 
 
 # entry
@@ -25,25 +26,16 @@ def get_universes():
 @app.route('/api/get_pred', methods=['POST'])
 def get_pred():
     fields = ['point_estimate', 'p_value', 'fit', 'stacking_weight',
-        'annotation']
-    fields = [app.schema[f] for f in fields if f in app.schema]
-    groups = group_by(fields, lambda x: x['file'])
-
-    header = []
-    res = None
-    for fn in groups:
-        df = pd.read_csv(os.path.join(app.data_folder, fn), na_filter=False)
-        names = ['uid'] + [d['name'] for d in groups[fn]]
-        cols = ['uid'] + [d['field'] for d in groups[fn]]
-        df = df[cols].rename(columns=dict(zip(cols, names)))
-        header.extend(names)
-        res = df if res is None else pd.merge(res, df, on='uid')
+        'annotation', 'standard_error']
+    res = common.read_results_batch(fields)
+    header = res.columns.tolist()
 
     # remove Inf and NA in point estimates
     res = remove_na(res, 'point_estimate', dtype=float)
 
     res = [res[n].values.tolist() for n in header]
-    reply = {'status': 'success', 'data': res, 'header': header}
+    reply = {'status': 'success', 'data': res, 'header': header,
+        'sensitivity': app.sensitivity}
     return jsonify(reply), 200
 
 # read uncertainty
@@ -77,8 +69,7 @@ def get_overview():
     res = {'schema': [app.schema[d]['name'] for d in app.schema],
         'decisions': app.decisions}
     res.update(app.visualizer)
-    reply = {'status': 'success', 'data': res, 
-        'sensitivity': app.sensitivity[app.visualizer['sensitivity']]}
+    reply = {'status': 'success', 'data': res}
     return jsonify(reply), 200
 
 # read the actual and predicted data of all data points in a universe
